import datetime
import operator
from pathlib import Path
from typing import List

import hydra
import omegaconf
import streamlit as st
import wandb
from hydra.core.global_hydra import GlobalHydra
from hydra.experimental import compose
from stqdm import stqdm

from nn_core.common import PROJECT_ROOT

WANDB_DIR: Path = PROJECT_ROOT / "wandb"
WANDB_DIR.mkdir(exist_ok=True, parents=True)

st_run_sel = st.sidebar


def local_checkpoint_selection(run_dir: Path, st_key: str) -> Path:
    checkpoint_paths: List[Path] = list(run_dir.rglob("checkpoints/*"))
    if len(checkpoint_paths) == 0:
        st.error(f"There's no checkpoint under {run_dir}! Are you sure the restore was successful?")
        st.stop()
    checkpoint_path: Path = st_run_sel.selectbox(
        label="Select a checkpoint",
        index=0,
        options=checkpoint_paths,
        format_func=operator.attrgetter("name"),
        key=f"checkpoint_select_{st_key}",
    )

    return checkpoint_path


def get_run_dir(entity: str, project: str, run_id: str) -> Path:
    """Get run directory.

    :param run_path: "entity/project/run_id"
    :return:
    """
    api = wandb.Api()
    run = api.run(path=f"{entity}/{project}/{run_id}")
    created_at: datetime = datetime.datetime.strptime(run.created_at, "%Y-%m-%dT%H:%M:%S")
    st.sidebar.markdown(body=f"[`Open on WandB`]({run.url})")

    timestamp: str = created_at.strftime("%Y%m%d_%H%M%S")

    matching_runs: List[Path] = [item for item in WANDB_DIR.iterdir() if item.is_dir() and item.name.endswith(run_id)]

    if len(matching_runs) > 1:
        st.error(f"More than one run matching unique id {run_id}! Are you sure about that?")
        st.stop()

    if len(matching_runs) == 1:
        return matching_runs[0]

    only_checkpoint: bool = st_run_sel.checkbox(label="Download only the checkpoint?", value=True)
    if st_run_sel.button(label="Download"):
        run_dir: Path = WANDB_DIR / f"restored-{timestamp}-{run.id}" / "files"
        files = [file for file in run.files() if "checkpoint" in file.name or not only_checkpoint]
        if len(files) == 0:
            st.error(f"There is no file to download from this run! Check on WandB: {run.url}")
        for file in stqdm(files, desc="Downloading files..."):
            file.download(root=run_dir)
        return run_dir
    else:
        st.stop()


def select_run_path(st_key: str, default_run_path: str):
    run_path: str = st_run_sel.text_input(
        label="Run path (entity/project/id):",
        value=default_run_path,
        key=f"run_path_select_{st_key}",
    )
    if not run_path:
        st.stop()
    tokens: List[str] = run_path.split("/")
    if len(tokens) != 3:
        st.error(f"This run path {run_path} doesn't look like a WandB run path! Are you sure about that?")
        st.stop()

    return tokens


def select_checkpoint(st_key: str = "MyAwesomeModel", default_run_path: str = ""):
    entity, project, run_id = select_run_path(st_key=st_key, default_run_path=default_run_path)

    run_dir: Path = get_run_dir(entity=entity, project=project, run_id=run_id)

    return local_checkpoint_selection(run_dir, st_key=st_key)


def get_hydra_cfg(config_name: str = "default") -> omegaconf.DictConfig:
    """Instantiate and return the hydra config -- streamlit and jupyter compatible.

    Args:
        config_name: .yaml configuration name, without the extension

    Returns:
        The desired omegaconf.DictConfig
    """
    GlobalHydra.instance().clear()
    hydra.experimental.initialize_config_dir(config_dir=str(PROJECT_ROOT / "conf"))
    return compose(config_name=config_name)



import logging
from pathlib import Path
from typing import Any, Dict, Optional

import pytorch_lightning as pl
from omegaconf import DictConfig
from pytorch_lightning import Callback, Trainer

from nn_core.model_logging import NNLogger
from nn_core.resume import parse_restore
from nn_core.serialization import METADATA_KEY, NNCheckpointIO

pylogger = logging.getLogger(__name__)


class NNTemplateCore(Callback):
    def __init__(self, restore_cfg: Optional[DictConfig]):
        self.resume_ckpt_path, self.resume_run_version = parse_restore(restore_cfg)
        self.restore_mode: Optional[str] = restore_cfg.get("mode", None) if restore_cfg is not None else None

    @property
    def resume_id(self) -> Optional[str]:
        return self.resume_run_version

    @property
    def trainer_ckpt_path(self) -> Optional[str]:
        return self.resume_ckpt_path if self.restore_mode != "finetune" else None

    @staticmethod
    def _is_nnlogger(trainer: Trainer) -> bool:
        return isinstance(trainer.logger, NNLogger)

    def on_train_start(self, trainer: pl.Trainer, pl_module: pl.LightningModule) -> None:
        if self._is_nnlogger(trainer):
            trainer.logger: NNLogger
            trainer.logger.upload_source()
            trainer.logger.log_configuration(model=pl_module)
            trainer.logger.watch_model(pl_module=pl_module)

    def setup(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule", stage: Optional[str] = None) -> None:
        if self.restore_mode == "finetune":
            checkpoint = NNCheckpointIO.load(path=Path(self.resume_ckpt_path))

            pl_module.load_state_dict(checkpoint["state_dict"])

    def on_train_end(self, trainer: pl.Trainer, pl_module: pl.LightningModule) -> None:
        if self._is_nnlogger(trainer):
            trainer.logger: NNLogger
            trainer.logger.upload_run_files()

    def on_save_checkpoint(
        self, trainer: pl.Trainer, pl_module: pl.LightningModule, checkpoint: Dict[str, Any]
    ) -> None:
        if self._is_nnlogger(trainer):
            trainer.logger.on_save_checkpoint(trainer=trainer, pl_module=pl_module, checkpoint=checkpoint)
            checkpoint[METADATA_KEY] = trainer.datamodule.metadata



import importlib
import inspect
import logging
import os
import shutil
import tempfile
import zipfile
from contextlib import contextmanager
from pathlib import Path
from typing import Any, Callable, Dict, Optional, Type, Union

import pytorch_lightning as pl
from pytorch_lightning.plugins import TorchCheckpointIO

METADATA_KEY: str = "metadata"

pylogger = logging.getLogger(__name__)


_METADATA_MODULE_KEY = f"{METADATA_KEY}_module"
_METADATA_CLASS_KEY = f"{METADATA_KEY}_class"


def _normalize_path(path: Union[Path, str]) -> Path:
    if isinstance(path, str):
        path = Path(path)
    return (path.parent / path.stem.split(".")[0]).with_suffix(".ckpt.zip")


class NNCheckpointIO(TorchCheckpointIO):
    def __init__(self, jailing_dir: Optional[str] = None):
        self.jailing_dir = jailing_dir

    @classmethod
    def load(cls, path: Path, map_location: Optional[Callable] = lambda storage, loc: storage):
        return cls().load_checkpoint(path=str(path), map_location=map_location)

    def save_checkpoint(self, checkpoint: Dict[str, Any], path, storage_options: Optional[Any] = None) -> None:
        checkpoint_dir: Path = _normalize_path(path=path)

        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_dir = Path(tmp_dir)

            if METADATA_KEY in checkpoint:
                metadata = checkpoint[METADATA_KEY]

                metadata_path: Path = tmp_dir / METADATA_KEY
                metadata_path.mkdir(exist_ok=True, parents=True)
                metadata.save(dst_path=metadata_path)

                checkpoint[_METADATA_MODULE_KEY] = inspect.getmodule(metadata).__name__
                checkpoint[_METADATA_CLASS_KEY] = type(metadata).__name__

                del checkpoint[METADATA_KEY]

            super().save_checkpoint(
                checkpoint=checkpoint, path=tmp_dir / "checkpoint.ckpt", storage_options=storage_options
            )

            compress_checkpoint(src_dir=tmp_dir, dst_file=checkpoint_dir)

    def load_checkpoint(self, path, map_location: Optional[Callable] = lambda storage, loc: storage) -> Dict[str, Any]:
        # load_checkpoint called from Trainer/Callbacks
        with extract_checkpoint(ckpt_file=Path(path)) as ckpt_dir:
            checkpoint = super().load_checkpoint(path=ckpt_dir / "checkpoint.ckpt", map_location=map_location)

            if _METADATA_MODULE_KEY in checkpoint:
                metadata_path: Path = ckpt_dir / METADATA_KEY
                if metadata_path.exists():
                    metadata_module = importlib.import_module(checkpoint[_METADATA_MODULE_KEY])
                    metadata = getattr(metadata_module, checkpoint[_METADATA_CLASS_KEY]).load(src_path=metadata_path)
                    checkpoint[METADATA_KEY] = metadata
                    del checkpoint[_METADATA_MODULE_KEY]
                    del checkpoint[_METADATA_CLASS_KEY]
                else:
                    raise FileNotFoundError(
                        "This checkpoint is corrupted. It appears data info is required but missing."
                    )

            return checkpoint

    def remove_checkpoint(self, path) -> None:
        if self.jailing_dir is None or path.startswith(self.jailing_dir):
            _normalize_path(path).unlink()
            pylogger.debug(f"Removing checkpoint from {path}")
        else:
            pylogger.debug(
                "Ignoring checkpoint deletion since it pertains to another run: "
                "https://github.com/PyTorchLightning/pytorch-lightning/issues/11379"
            )


def compress_checkpoint(src_dir: Path, dst_file: Path, delete_dir: bool = True):

    with zipfile.ZipFile(_normalize_path(dst_file), "w") as zip_file:
        for folder, subfolders, files in os.walk(src_dir):
            folder: Path = Path(folder)
            for file in files:
                zip_file.write(
                    folder / file,
                    os.path.relpath(os.path.join(folder, file), src_dir),
                    compress_type=zipfile.ZIP_DEFLATED,
                )

    if delete_dir:
        pylogger.debug(f"Deleting the checkpoint folder: '{src_dir}'")
        shutil.rmtree(path=src_dir, ignore_errors=True)


@contextmanager
def extract_checkpoint(ckpt_file: Path) -> Path:
    with tempfile.TemporaryDirectory() as tmp_dir:
        pylogger.debug(f"Extracting archive file '{ckpt_file}' to temp dir '{tmp_dir}'")
        with zipfile.ZipFile(_normalize_path(ckpt_file), "r") as compressed_ckpt:
            compressed_ckpt.extractall(tmp_dir)
        yield Path(tmp_dir)


def load_model(module_class: Type[pl.LightningModule], checkpoint_path: Path):
    checkpoint = NNCheckpointIO.load(path=checkpoint_path)

    model = module_class._load_model_state(checkpoint=checkpoint, metadata=checkpoint["metadata"])
    return model



import logging
import os
from contextlib import contextmanager
from typing import List, Optional

import dotenv
import numpy as np
from hydra.core.hydra_config import HydraConfig
from omegaconf import DictConfig
from pytorch_lightning import seed_everything
from rich.prompt import Prompt

pylogger = logging.getLogger(__name__)


def get_env(env_name: str, default: Optional[str] = None) -> str:
    """Safely read an environment variable.

    Raises errors if it is not defined or it is empty.

    :param env_name: the name of the environment variable
    :param default: the default (optional) value for the environment variable

    :return: the value of the environment variable
    """
    if env_name not in os.environ:
        if default is None:
            message = f"{env_name} not defined and no default value is present!"
            pylogger.error(message)
            raise KeyError(message)
        return default

    env_value: str = os.environ[env_name]
    if not env_value:
        if default is None:
            message = f"{env_name} has yet to be configured and no default value is present!"
            pylogger.error(message)
            raise ValueError(message)
        return default

    return env_value


def load_envs(env_file: Optional[str] = None) -> None:
    """Load all the environment variables defined in the `env_file`.

    This is equivalent to `. env_file` in bash.

    It is possible to define all the system specific variables in the `env_file`.

    :param env_file: the file that defines the environment variables to use. If None
                     it searches for a `.env` file in the project.
    """
    dotenv.load_dotenv(dotenv_path=env_file, override=True)


@contextmanager
def environ(**kwargs):
    """Temporarily set the process environment variables.

    https://stackoverflow.com/a/34333710

    >>> with environ(PLUGINS_DIR=u'test/plugins'):
    ...   "PLUGINS_DIR" in os.environ
    True

    >>> "PLUGINS_DIR" in os.environ
    False

    :type kwargs: dict[str, unicode]
    :param kwargs: Environment variables to set
    """
    old_environ = dict(os.environ)
    os.environ.update(kwargs)
    try:
        yield
    finally:
        os.environ.clear()
        os.environ.update(old_environ)


def enforce_tags(tags: Optional[List[str]]) -> List[str]:
    if tags is None:
        if "id" in HydraConfig().cfg.hydra.job:
            # We are in multi-run setting (either via a sweep or a scheduler)
            message: str = "You need to specify 'core.tags' in a multi-run setting!"
            pylogger.error(message)
            raise ValueError(message)

        pylogger.warning("No tags provided, asking for tags...")
        tags = Prompt.ask("Enter a list of comma separated tags", default="develop")
        tags = [x.strip() for x in tags.split(",")]

    pylogger.info(f"Tags: {tags if tags is not None else []}")
    return tags


def seed_index_everything(train_cfg: DictConfig, sampling_seed: int = 42) -> Optional[int]:
    if "seed_index" in train_cfg and train_cfg.seed_index is not None:
        seed_index = train_cfg.seed_index
        np.random.seed(sampling_seed)
        seeds = np.random.randint(np.iinfo(np.int32).max, size=max(42, seed_index + 1))
        seed = seeds[seed_index]
        seed_everything(seed)
        pylogger.info(f"Setting seed {seed} from seeds[{seed_index}]")
        return seed
    else:
        pylogger.warning("The seed has not been set! The reproducibility is not guaranteed.")
        return None



import logging
import os
from pathlib import Path

import git

from nn_core.common.utils import load_envs

logger = logging.getLogger(__name__)


# Load environment variables
load_envs()

try:
    PROJECT_ROOT = Path(git.Repo(Path.cwd(), search_parent_directories=True).working_dir)
except git.exc.InvalidGitRepositoryError:
    PROJECT_ROOT = Path.cwd()

logger.debug(f"Inferred project root: {PROJECT_ROOT}")
os.environ["PROJECT_ROOT"] = str(PROJECT_ROOT)



import argparse
import logging
import os
import shutil
from pathlib import Path
from typing import Any, Dict, Optional, Union

import hydra
import pytorch_lightning
from omegaconf import DictConfig, OmegaConf
from pytorch_lightning import LightningModule, Trainer
from pytorch_lightning.callbacks import ModelCheckpoint
from pytorch_lightning.loggers import LightningLoggerBase

from nn_core.common import PROJECT_ROOT

pylogger = logging.getLogger(__name__)


_STATS_KEY: str = "stats"


class NNLogger(LightningLoggerBase):

    __doc__ = LightningLoggerBase.__doc__

    def __init__(self, logging_cfg: DictConfig, cfg: DictConfig, resume_id: Optional[str]):
        super().__init__()
        self.logging_cfg = logging_cfg
        self.cfg = cfg
        self.resume_id = resume_id

        self.storage_dir: str = cfg.core.storage_dir
        self.wandb: bool = self.logging_cfg.logger["_target_"].endswith("WandbLogger")

        if self.cfg.train.trainer.fast_dev_run and self.wandb:
            # Switch wandb mode to offline to prevent online logging
            pylogger.info("Setting the logger in 'offline' mode")
            self.logging_cfg.logger.mode = "offline"

        pylogger.info(f"Instantiating <{self.logging_cfg.logger['_target_'].split('.')[-1]}>")
        self.wrapped: LightningLoggerBase = hydra.utils.instantiate(self.logging_cfg.logger, version=self.resume_id)

        # force experiment lazy initialization
        _ = self.wrapped.experiment

    def __getattr__(self, item: str) -> Any:
        if self.wrapped is not None:
            pylogger.debug(f"Delegation with '__getattr__': {self.wrapped.__class__.__qualname__}.{item}")
            return getattr(self.wrapped, item)

    def watch_model(self, pl_module: LightningModule):
        if self.wandb and "wandb_watch" in self.logging_cfg:
            pylogger.info("Starting to 'watch' the module")
            self.wrapped.watch(pl_module, **self.logging_cfg["wandb_watch"])

    def upload_source(self) -> None:
        if self.logging_cfg.upload.source and self.wandb:
            pylogger.info("Uploading source code to W&B")
            self.wrapped.experiment.log_code(
                root=PROJECT_ROOT,
                name=None,
                include_fn=(
                    lambda path: path.startswith(
                        (
                            str(PROJECT_ROOT / "conf"),
                            str(PROJECT_ROOT / "src"),
                            str(PROJECT_ROOT / "setup.cfg"),
                            str(PROJECT_ROOT / "env.yaml"),
                        )
                    )
                    and path.endswith((".py", ".yaml", ".yml", ".toml", ".cfg"))
                ),
            )

    def on_save_checkpoint(self, trainer: Trainer, pl_module: LightningModule, checkpoint: Dict[str, Any]) -> None:
        # Attach to each checkpoint saved the configuration and the wandb run path (to resume logging from
        # only the checkpoint)
        pylogger.debug("Attaching 'cfg' to the checkpoint")
        checkpoint["cfg"] = OmegaConf.to_container(trainer.logger.cfg, resolve=True)

        pylogger.debug("Attaching 'run_path' to the checkpoint")
        checkpoint[
            "run_path"
        ] = f"{trainer.logger.experiment.entity}/{trainer.logger.experiment.project_name()}/{trainer.logger.version}"

    def after_save_checkpoint(self, checkpoint_callback: ModelCheckpoint) -> None:
        # Log the checkpoint meta information
        self.add_path(obj_id="checkpoints/best", obj_path=checkpoint_callback.best_model_path)
        self.add_path(
            obj_id="checkpoints/best_score",
            obj_path=str(checkpoint_callback.best_model_score.detach().cpu().item()),
        )

    def add_path(self, obj_id: str, obj_path: str) -> None:
        key = f"paths/{obj_id}"
        pylogger.debug(f"Logging '{key}'")
        self.experiment.config.update({key: str(obj_path)}, allow_val_change=True)

    @property
    def save_dir(self) -> Optional[str]:
        return self.storage_dir

    @property
    def experiment(self) -> Any:
        """Return the experiment object associated with this logger."""
        return self.wrapped.experiment

    def log_metrics(self, metrics: Dict[str, float], step: Optional[int] = None):
        """Records metrics.

        This method logs metrics as as soon as it received them. If you want to aggregate
        metrics for one specific `step`, use the
        :meth:`~pytorch_lightning.loggers.base.LightningLoggerBase.agg_and_log_metrics` method.

        Args:
            metrics: Dictionary with metric names as keys and measured quantities as values
            step: Step number at which the metrics should be recorded
        """
        return self.wrapped.log_metrics(metrics=metrics, step=step)

    def log_hyperparams(self, params: argparse.Namespace, *args, **kwargs):
        """Record hyperparameters.

        Args:
            params: :class:`~argparse.Namespace` containing the hyperparameters
            args: Optional positional arguments, depends on the specific logger being used
            kwargs: Optional keywoard arguments, depends on the specific logger being used
        """
        raise RuntimeError(
            "This method is called automatically by PyTorch Lightning if save_hyperparameters(logger=True) is called. "
            "The whole configuration is already logged by logger.log_configuration, set logger=False"
        )

    def log_text(self, *args, **kwargs) -> None:
        """Log text.

        Arguments are directly passed to the logger.
        """
        return self.wrapped.log_text(*args, **kwargs)

    def log_image(self, *args, **kwargs) -> None:
        """Log image.

        Arguments are directly passed to the logger.
        """
        return self.wrapped.log_image(*args, **kwargs)

    @property
    def name(self) -> str:
        """Return the experiment name."""
        return self.wrapped.name

    @property
    def version(self) -> Union[int, str]:
        """Return the experiment version."""
        return self.wrapped.version

    @property
    def run_dir(self) -> str:
        # TODO: verify remote URLs handling
        return os.path.join(*map(str, (self.storage_dir, self.name, self.version)))

    def log_configuration(
        self,
        model: pytorch_lightning.LightningModule,
        cfg: Union[Dict[str, Any], argparse.Namespace, DictConfig] = None,
        *args,
        **kwargs,
    ):
        """This method controls which parameters from Hydra config are saved by Lightning loggers.

        Additionally saves:
            - sizes of train, val, test dataset
            - number of trainable model parameters

        Args:
            cfg (DictConfig): [description]
            model (pl.LightningModule): [description]
            trainer (pl.Trainer): [description]
        """
        if cfg is None:
            cfg = OmegaConf.create(self.cfg)

        if isinstance(cfg, DictConfig):
            cfg: Union[Dict[str, Any], argparse.Namespace, DictConfig] = OmegaConf.to_container(cfg, resolve=True)

        # Store the YaML config separately into the wandb dir
        yaml_conf: str = OmegaConf.to_yaml(cfg=cfg)
        run_dir: Path = Path(self.run_dir)
        run_dir.mkdir(exist_ok=True, parents=True)
        config_save_path = run_dir / "config.yaml"
        pylogger.debug(f"Saving the configuration in: {config_save_path}")
        config_save_path.write_text(yaml_conf)

        # save number of model parameters
        pylogger.debug("Injecting model statistics in the 'cfg'")
        cfg[f"{_STATS_KEY}/params_total"] = sum(p.numel() for p in model.parameters())
        cfg[f"{_STATS_KEY}/params_trainable"] = sum(p.numel() for p in model.parameters() if p.requires_grad)
        cfg[f"{_STATS_KEY}/params_not_trainable"] = sum(p.numel() for p in model.parameters() if not p.requires_grad)

        # send hparams to all loggers
        pylogger.debug("Logging 'cfg'")
        self.wrapped.log_hyperparams(cfg)

    def upload_run_files(self):
        if self.logging_cfg.upload.run_files:
            if self.wandb:
                pylogger.info("Uploading run files to W&B")
                shutil.copytree(self.run_dir, f"{self.wrapped.experiment.dir}/run_files")

                # FIXME: symlink not working for some reason
                # os.symlink(self.run_dir, f"{self.wrapped.experiment.dir}/run_files", target_is_directory=True)



import logging
import re
import tempfile
from operator import xor
from pathlib import Path
from typing import Optional, Tuple

import wandb
from omegaconf import DictConfig
from wandb.apis.public import Run

from nn_core.common import PROJECT_ROOT
from nn_core.common.utils import environ
from nn_core.serialization import NNCheckpointIO

pylogger = logging.getLogger(__name__)

RUN_PATH_PATTERN = re.compile(r"^([^/]+)/([^/]+)/([^/]+)$")

RESUME_MODES = {
    None: {
        "logging": False,
        "trainer": False,
        "weights": False,
    },
    "finetune": {
        "logging": False,
        "trainer": False,
        "weights": True,
    },
    "hotstart": {
        "logging": False,
        "trainer": True,
        "weights": True,
    },
    "continue": {
        "logging": True,
        "trainer": True,
        "weights": True,
    },
}


def resolve_ckpt(ckpt_or_run_path: str) -> str:
    """Resolve the run path or ckpt to a checkpoint.

    Args:
        ckpt_or_run_path: run identifier or checkpoint path

    Returns:
        an existing path towards the best checkpoint
    """
    if RUN_PATH_PATTERN.match(ckpt_or_run_path):
        # If WANDB_DIR is set (as it is the case with our hydra configuration), the run dir is created by wandb in the
        # project's root folder instead of in a temp directory.
        with tempfile.TemporaryDirectory() as tmp_dir, environ(WANDB_DIR=tmp_dir):
            # We are resolving the path from a wandb run id
            try:
                api = wandb.Api()
                run: Run = api.run(path=ckpt_or_run_path)
                ckpt_or_run_path = run.config["paths/checkpoints/best"]
                return ckpt_or_run_path
            except wandb.errors.CommError:
                raise ValueError(f"Checkpoint or run not found: {ckpt_or_run_path}")

    _ckpt_or_run_path: Path = Path(ckpt_or_run_path)
    # If the path is relative, it is wrt the PROJECT_ROOT, so it is prepended.
    if not _ckpt_or_run_path.is_absolute():
        _ckpt_or_run_path = PROJECT_ROOT / _ckpt_or_run_path

    if _ckpt_or_run_path.exists():
        return ckpt_or_run_path


def resolve_run_path(ckpt_or_run_path: str) -> str:
    """Resolve the run path or ckpt to a run path.

    Args:
        ckpt_or_run_path: run identifier or checkpoint path

    Returns:
        an wandb run path identifier
    """
    if RUN_PATH_PATTERN.match(ckpt_or_run_path):
        return ckpt_or_run_path

    try:
        return NNCheckpointIO.load(path=Path(ckpt_or_run_path))["run_path"]
    except FileNotFoundError:
        raise ValueError(f"Checkpoint or run not found: {ckpt_or_run_path}")


def resolve_run_version(ckpt_or_run_path: Optional[str] = None, run_path: Optional[str] = None) -> str:
    """Resolve the run path or ckpt to the wandb run version.

    Args:
        ckpt_or_run_path: run identifier or checkpoint path
        run_path: the run path if already available

    Returns:
        a wandb run version
    """
    if run_path is None:
        run_path = resolve_run_path(ckpt_or_run_path)
    return RUN_PATH_PATTERN.match(run_path).group(3)


# TODO: Refactor returning type to include restore mode too.
def parse_restore(restore_cfg: DictConfig) -> Tuple[Optional[str], Optional[str]]:
    if restore_cfg is None:
        return None, None

    ckpt_or_run_path = restore_cfg.ckpt_or_run_path
    resume_mode = restore_cfg.mode

    resume_ckpt_path = None
    resume_run_version = None

    if xor(bool(ckpt_or_run_path), bool(resume_mode)):
        pylogger.warning(f"Inconsistent resume modality {resume_mode} and checkpoint path '{ckpt_or_run_path}'")
    else:
        pylogger.info(f"Restoring with mode: <{resume_mode}>")

    if resume_mode not in RESUME_MODES:
        message = f"Unsupported resume mode {resume_mode}. Available resume modes are: {RESUME_MODES}"
        pylogger.error(message)
        raise ValueError(message)

    flags = RESUME_MODES[resume_mode]

    if ckpt_or_run_path is not None:
        resume_ckpt_path = resolve_ckpt(ckpt_or_run_path)
        pylogger.info(f"Resolved checkpoint path: '{resume_ckpt_path}'")

        if flags["logging"]:
            run_path = resolve_run_path(ckpt_or_run_path)
            resume_run_version = resolve_run_version(run_path=run_path)
            pylogger.info(f"Resume logging to: '{run_path}'")

    return resume_ckpt_path, resume_run_version



import logging
from datetime import datetime
from typing import Optional

from rich.console import ConsoleRenderable
from rich.logging import RichHandler
from rich.traceback import Traceback


class NNRichHandler(RichHandler):
    def render(
        self,
        *,
        record: logging.LogRecord,
        traceback: Optional[Traceback],
        message_renderable: ConsoleRenderable,
    ) -> ConsoleRenderable:
        # Hack to display the logger name instead of the filename in the rich logs
        path = record.name  # str(Path(record.pathname))
        level = self.get_level_text(record)
        time_format = None if self.formatter is None else self.formatter.datefmt
        log_time = datetime.fromtimestamp(record.created)

        log_renderable = self._log_render(
            self.console,
            [message_renderable] if not traceback else [message_renderable, traceback],
            log_time=log_time,
            time_format=time_format,
            level=level,
            path=path,
            line_no=record.lineno,
            link_path=record.pathname if self.enable_link_path else None,
        )
        return log_renderable
